/*
 * Copyright (C) 2022  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "fred_api/registry/domain/service_domain_grpc.pb.h"
#include "fred_api/registry/domain/service_domain_grpc.grpc.pb.h"

#include <google/protobuf/util/json_util.h>

#include <grpc/grpc.h>
#include <grpcpp/channel.h>
#include <grpcpp/client_context.h>
#include <grpcpp/create_channel.h>
#include <grpcpp/security/credentials.h>

#include <cstdlib>
#include <iostream>
#include <stdexcept>

namespace {

std::string to_json(const google::protobuf::Message& msg)
{
    std::string json;
    google::protobuf::util::JsonPrintOptions options;
    options.add_whitespace = true;
    options.preserve_proto_field_names = true;
    const auto status = google::protobuf::util::MessageToJsonString(msg, &json, options);
    if (status.ok())
    {
        return json;
    }
    throw std::runtime_error{"unable convert to json"};
}

}//namespace {anonymous}

int main(int argc, char* argv[])
{
    if (argc != 3)
    {
        std::cerr << "bad number of arguments; expect two arguments\n"
                     "usage: " << argv[0] << " <grpc server endpoint> <request in json format>" << std::endl;
        return EXIT_FAILURE;
    }
    grpc::ClientContext context;
    auto channel = grpc::CreateChannel(argv[1], grpc::InsecureChannelCredentials());
    auto stub = Fred::Registry::Api::Domain::Domain::NewStub(channel);
    Fred::Registry::Api::Domain::DomainsByContactRequest request;
    {
        const auto status = google::protobuf::util::JsonStringToMessage(argv[2], &request);
        if (!status.ok())
        {
            std::cerr << "not a json: " << status.error_message() << std::endl;
            return EXIT_FAILURE;
        }
    }
    Fred::Registry::Api::Domain::DomainsByContactReply reply;
    std::unique_ptr<grpc::ClientReader<Fred::Registry::Api::Domain::DomainsByContactReply>> reader(
            stub->get_domains_by_contact(&context, request));
    while (reader->Read(&reply))
    {
        std::cout << to_json(reply) << std::endl;
    }
    const auto status = reader->Finish();
    if (status.ok())
    {
        return EXIT_SUCCESS;
    }
    std::cerr << "failure: " << status.error_message() << std::endl;
    return EXIT_FAILURE;
}
